﻿using System.Windows.Controls;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Data;
using SilverlightApplication1.ServiceReference1;

namespace SilverlightApplication1
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();

            var context = new NorthwindServiceSoapClient();

            var view = new VirtualQueryableCollectionView<Customer>() { LoadSize = 10, VirtualItemCount = 100 };
            view.FilterDescriptors.CollectionChanged += (s, e) =>
                {
                    if (view.FilterDescriptors.Count == 0)
                    {
                        view.VirtualItemCount = 100;
                    }
                };

            bool shouldLoad = true;
            view.ItemsLoading += (s, e) =>
            {
                if (!shouldLoad)
                {
                    shouldLoad = true;
                    return;
                }

                context.GetCustomersAsync(e.StartIndex, e.ItemCount,
                               view.SortDescriptors.ToDynamicLinq(),
                               view.FilterDescriptors.ToDynamicLinq(),
                               view.GroupDescriptors.ToDynamicLinq(), e.StartIndex);
            };

            context.GetCustomersCompleted += (s, e) =>
            {
                if (e.Result.Count != -1 && e.Result.Count != view.VirtualItemCount)
                {
                    shouldLoad = false;

                    view.VirtualItemCount = e.Result.Count;
                }

                view.Load((int)e.UserState, e.Result.Data);
            };

            DataContext = view;
        }

        void RadGridView1_DistinctValuesLoading(object sender, GridViewDistinctValuesLoadingEventArgs e)
        {
            var context = new NorthwindServiceSoapClient();
            var collection = new RadObservableCollection<object>();

            e.ItemsSource = collection;

            context.GetDistinctValuesCompleted += (s, args) =>
            {
                collection.Clear();
                collection.AddRange(args.Result);
            };

            context.GetDistinctValuesAsync(e.Column.UniqueName);
        }

        private void RadGridView_AutoGeneratingColumn(object sender, Telerik.Windows.Controls.GridViewAutoGeneratingColumnEventArgs e)
        {
            e.Column.ShowFieldFilters = false;
        }
    }
}